@@ -18,6 +18,8 @@ namespace unittest
 #include <thrust/device_new.h>
 #include <thrust/device_delete.h>
 
+#include <thrust/detail/nv_target.h>
+
 #if THRUST_DEVICE_SYSTEM == THRUST_DEVICE_SYSTEM_CUDA || THRUST_DEVICE_SYSTEM == THRUST_DEVICE_SYSTEM_HIP
 
 #define ASSERT_STATIC_ASSERT(X)                                                                      \
@@ -113,11 +115,9 @@ namespace unittest
         {
             static_assert_exception ex(filename, lineno);
 
-#if defined(__CUDA_ARCH__) || defined(__HIP_DEVICE_COMPILE__)
-            *detail::device_exception = ex;
-#else
-            throw ex;
-#endif
+            NV_IF_TARGET(NV_IS_DEVICE,
+                         (*detail::device_exception = ex;),
+                         (throw ex;));
         }
     }
 }
